from typing import Literal

import typer

from .config import (
    configure_database,
    configure_module,
    configure_project,
    configure_user,
    create_bife_config,
    install_packages,
)

typer_app = typer.Typer()
new = typer.Typer()
typer_app.add_typer(new, name='new')


@new.command()
def module(name: str):
    """
    Create a new module inside your fastapi project
    """
    print(f'Hello {name}')


@new.command()
def app(project_name: str):
    """
    Create a new app with FastAPI!
    """
    database_config = configure_database()
    project_config = configure_project()
    user_config = configure_user()
    module_config = configure_module()
    create_bife_config(module_config.module_path, project_config.structure)
    if project_config.structure == 'default':
        from .structs.default.generate import DEPENDENCIES, run

        run(project_name, module_config, user_config, database_config)
        install_packages(DEPENDENCIES)


def start():
    typer_app()



from fastapi import APIRouter, Depends, HTTPException, Response, status
from libs.security.security import Security

from .events import UserEvents
from .ext import (
    InvalidEmail,
    InvalidGender,
    InvalidName,
    InvalidPassword,
    InvalidUserLogin,
    InvalidUsername,
    UserAlreadyExists,
    UserNotFound,
)
from .models import UserModel
from .repository import UserRepository
from .schemas import LoginUserRequest, RegisterUserRequest, UserAuthResponse
from .service import UserService

router: APIRouter = APIRouter(tags=['user'], prefix='/user')
security: Security = Security()

repository: UserRepository = UserRepository()
service: UserService = UserService()
events: UserEvents = UserEvents()


@router.post('/login', response_model=UserAuthResponse)
def user_login(data: LoginUserRequest, response: Response):
    try:
        user = repository.login_user(data)
    except UserNotFound as e:
        raise HTTPException(status.HTTP_404_NOT_FOUND, e.args[0])
    except (InvalidPassword, InvalidUserLogin) as e:
        raise HTTPException(status.HTTP_400_BAD_REQUEST, e.args[0])

    tokens = service.generate_access_token(user.uuid)
    response.set_cookie('access_token', tokens['access_token'])
    response.set_cookie('refresh_access_token', tokens['refresh_access_token'])
    return UserAuthResponse(**tokens)


@router.post('/register', response_model=UserAuthResponse)
def register_user(data: RegisterUserRequest, response: Response):
    try:
        user = repository.register_user(data)
    except (
        InvalidUsername,
        InvalidEmail,
        InvalidGender,
        InvalidName,
        InvalidPassword,
    ) as e:
        raise HTTPException(status.HTTP_400_BAD_REQUEST, e.args[0])
    except UserAlreadyExists as e:
        raise HTTPException(status.HTTP_409_CONFLICT, e.args[0])
    tokens = service.generate_access_token(user.uuid)
    response.set_cookie('access_token', tokens['access_token'])
    response.set_cookie('refresh_access_token', tokens['refresh_access_token'])
    return UserAuthResponse(**tokens)


@router.get('/me')
def get_about_user(
    current_user: UserModel = Depends(security.get_current_user),
):
    return current_user.__dict__



import os

from dotenv import find_dotenv, load_dotenv
from pydantic import BaseModel

load_dotenv(find_dotenv())


class Config(BaseModel):
    # API
    API_PORT: int = int(os.getenv('API_PORT', 8000))
    API_HOST: str = os.getenv('API_HOST', '0.0.0.0')

    # Database
    DATABASE_CONNECTION: str = os.getenv(
        'DATABASE_CONNECTION', 'sqlite:///database.sqlite'
    )

    # JWt
    JWT_SECRET_KEY: str = os.getenv('JWT_SECRET_KEY', 'secret')
    JWT_REFRESH_SECRET_KEY: str = os.getenv('JWT_REFRESH_SECRET_KEY', 'secret')
    JWT_ALGORITHM: str = os.getenv('JWT_ALGORITHM', 'HS256')
    ACCESS_TOKEN_EXPIRE_MINUTES: int = int(
        os.getenv('ACCESS_TOKEN_EXPIRE_MINUTES', default=30)
    )  # 30 minutes
    REFRESH_TOKEN_EXPIRE_MINUTES: int = int(
        os.getenv('REFRESH_TOKEN_EXPIRE_MINUTES', default=10080)
    )  # 7 days


def get_config() -> Config:
    return Config()



from pydantic import BaseModel, Field


# User login
class LoginUserRequest(BaseModel):
    user: str = Field(example='johndoe@email.com')
    password: str = Field(example='Pass123')


class RegisterUserRequest(BaseModel):
    fullname: str = Field(example='John Doe')
    username: str = Field(example='john_doe')
    email: str = Field(example='johndoe@email.com')
    password: str = Field(example='Pass123')
    gender: str = Field(example='woman')
    # verification: str = Field(example='...')


class UserAuthResponse(BaseModel):
    access_token: str = Field(example='Bearer ...')
    refresh_access_token: str = Field(example='Bearer ...')



import uuid
from datetime import datetime

from libs.database import Base
from sqlalchemy import ForeignKey, String
from sqlalchemy.orm import Mapped, mapped_column


class UserModel(Base):

    __tablename__ = 'users'

    id: Mapped[int] = mapped_column(init=False, primary_key=True)
    uuid: Mapped[str] = mapped_column(
        String(60), init=False, default_factory=lambda: str(uuid.uuid4())
    )

    username: Mapped[str] = mapped_column(String(60), nullable=False)
    fullname: Mapped[str] = mapped_column(String(60), nullable=False)
    password: Mapped[str] = mapped_column(String(256), nullable=False)
    email: Mapped[str] = mapped_column(String(60), nullable=False)

    is_admin: Mapped[bool] = mapped_column(default=False)
    disabled: Mapped[bool] = mapped_column(default=False)

    member_at: Mapped[datetime] = mapped_column(
        default_factory=lambda: datetime.now()
    )
    modified_at: Mapped[datetime] = mapped_column(
        default_factory=lambda: datetime.now()
    )



import os
import shutil
import sys

from mako.template import Template

from ...config import DatabaseConfig, ModuleConfig, UserConfig

DEPENDENCIES = [
    'fastapi',
    'pyjwt',
    'argon2-cffi',
    'sqlalchemy',
    'python-dotenv',
    'uvicorn',
]


def run(
    project_name: str,
    module_config: ModuleConfig,
    user_config: UserConfig,
    database_config: DatabaseConfig,
):
    current_dir = os.path.dirname(__file__)
    os.makedirs(project_name)
    copy_file_to_project(['app.py', 'main.py', 'config.py'], project_name)
    copy_directorys_to_project(['libs', 'utils'], project_name)
    copy_directorys_to_project(
        [os.path.join('modules', 'user')],
        os.path.join(project_name, module_config.module_path),
    )
    update_env_file(project_name)
    generate_bootstrap(project_name)
    generate_security_module(project_name, module_config.module_path)


def update_env_file(project_name: str):
    current_dir = os.path.dirname(__file__)
    env_file_template = os.path.join(current_dir, '.env.template')

    if not os.path.isfile('.env'):
        with open('.env', 'w') as file:
            file.write('')

    with open(env_file_template, 'r') as file:
        template = Template(file.read())
        content = template.render(project_name=project_name)

    with open('.env', 'a') as file:
        file.write(f'\n{content}')


def create_required_directorys(prefix: str, paths: list[str]):
    for path in paths:
        os.makedirs(os.path.join(prefix, path))


def copy_file_to_project(files: list[str], path: str):
    current_dir = os.path.dirname(__file__)
    for file in files:
        shutil.copy(os.path.join(current_dir, file), path)


def copy_directorys_to_project(directorys: list[str], path: str):
    current_dir = os.path.dirname(__file__)
    for directory in directorys:
        shutil.copytree(
            os.path.join(current_dir, directory),
            os.path.join(path, directory.split('/')[-1]),
        )


def generate_security_module(project_name: str, module_path: str):
    security_path_template = os.path.join(
        project_name, 'libs', 'security', 'security.template'
    )
    security_path_py = os.path.join(
        project_name, 'libs', 'security', 'security.py'
    )
    with open(security_path_template, 'r') as file:
        template = file.read()
    import_string = module_path.replace('/', '.')
    if import_string.endswith('.'):
        import_string = import_string[:-1]
    security_module = Template(template).render(module_path=import_string)
    with open(security_path_template, 'w') as file:
        file.write(security_module)
    os.rename(security_path_template, security_path_py)


def generate_bootstrap(project_name: str):
    current_dir = os.path.dirname(__file__)
    template = Template(
        open(
            os.path.join(current_dir, 'libs', 'bootstrap', 'bootstrap.py')
        ).read()
    )
    boostrap_content = template.render(project_name=project_name)
    with open(
        os.path.join(project_name, 'libs', 'bootstrap', 'bootstrap.py'), 'w'
    ) as file:
        file.write(boostrap_content)


import os
import subprocess

import toml

# def check_is_poetry():
#     if not os.path.isfile('pyproject.toml'):
#         print('pyproject.toml not found')
#         return False
#     try:
#         with open('pyproject.toml', 'r') as file:
#             toml_content = toml.load(file)
#         if 'tool' in toml_content and 'poetry' in toml_content['tool']:
#             dependencies = toml_content['tool']['poetry'].get('dependencies', None)
#             if dependencies:
#                 packages = ' '.join([f"{package}{version}" for package, version in dependencies.items()])
#                 subprocess.run(['poetry', 'add', packages], check=True)
#                 print("Dependências adicionadas com sucesso usando Poetry.")
#                 return

#         raise ValueError("Arquivo 'pyproject.toml' não parece ser do Poetry.")

#     except Exception as e:
#         print(f"Erro ao processar 'pyproject.toml': {e}")
#         print("Instalando dependências usando pip...")
#         subprocess.run(['pip', 'install', '-r', 'requirements.txt'], check=True)
#         print("Dependências instaladas com sucesso usando pip.")


def new_module(module_name: str):
    current_dir = os.path.dirname(__file__)
    # full_path = os.path.abspath(path)

    # if not os.path.isdir(full_path):
    #     print(f'Invalid path: {full_path}')
    #     sys.exit(1)

    # module_path = os.path.join(full_path, module_name)

    # if os.path.isdir(module_path):
    #     print(f'Path {module_path} already exists')
    #     sys.exit(1)

    # os.makedirs(module_path)

    # templates_dir = os.path.join(current_dir, 'general')
    # templates = {
    #     'controller': 'controller.template',
    #     'models': 'models.template',
    #     'repository': 'repository.template',
    #     'schemas': 'schemas.template'
    # }

    # for template_name, template_file in templates.items():
    #     template_content = Template(open(os.path.join(templates_dir, template_file)).read()).render(
    #         module_name=module_name,
    #         module_name_capital=module_name.capitalize()
    #     )
    #     with open(os.path.join(module_path, f'{template_name}.py'), 'w') as file:
    #         file.write(template_content)



import os
import subprocess
from dataclasses import dataclass, field
from subprocess import PIPE
from typing import Literal

import questionary
from mako.template import Template
from questionary import Choice


@dataclass
class DatabaseConfig:
    orm: Literal['sqlalchemy', 'sqlmodel'] = field(default='sqlalchemy')
    use_alembic: bool = field(default=True)


@dataclass
class UserConfig:
    create_module: bool = field(default=True)
    store_jwt: Literal['headers', 'cookie', 'both', None] = field(
        default='headers'
    )


@dataclass
class ModuleConfig:
    module_path: str = field(default='modules/')


@dataclass
class ProjectConfig:
    structure: str = field(default='default')


def configure_database() -> DatabaseConfig:
    """
    Ask database configuration
    """
    orm = questionary.select(
        '💾 Do you want use Sqlalchemy or Sqlmodel?',
        choices=[
            Choice('Sqlalchemy', 'Sqlmodel'),
            Choice('Sqlmodel', 'sqlmodel'),
        ],
    ).ask()

    alembic = questionary.select(
        '📝 Do you want use alembic?',
        choices=[Choice('Yes', True), Choice('No', False)],
    ).ask()

    database_config = DatabaseConfig(orm, alembic)
    return database_config


def configure_user() -> UserConfig:
    """
    Ask user configuration
    """
    print(' ')
    create_user_module: bool = questionary.select(
        '👤 Create a user module?',
        choices=[Choice('Yes', True), Choice('No', False)],
    ).ask()
    if not create_user_module:
        return UserConfig(False, None)
    store_jwt: Literal['headers', 'cookie', 'both'] = questionary.select(
        '🔐 Where you want to get JWT token?',
        choices=[
            Choice('Headers', 'headers'),
            Choice('Cookies', 'cookies'),
            Choice('Both', 'both'),
        ],
    ).ask()
    user_config = UserConfig(create_user_module, store_jwt)
    return user_config


def configure_module() -> ModuleConfig:
    """
    Ask user configuration
    """
    print(' ')

    module_path: str = questionary.select(
        '📁 Where you want store modules?', choices=['modules/', 'modules/api/']
    ).ask()
    module_config = ModuleConfig(module_path)
    return module_config


def configure_project() -> ProjectConfig:
    """
    Ask project structure
    """
    print(' ')

    project_config: str = questionary.select(
        '🗄️ Select project structure', choices=['default']
    ).ask()
    structure_config = ProjectConfig(project_config)
    return structure_config


def create_bife_config(modules_path: str, project_structure: str):
    current_dir = os.path.dirname(__file__)
    with open(os.path.join(current_dir, 'config.template'), 'r') as file:
        template_content = file.read()
    bife_config_content = Template(template_content).render(
        modules_path=modules_path, project_structure=project_structure
    )
    with open('bife.config', 'w') as file:
        file.write(bife_config_content)


def install_packages(dependencies: list[str]):
    while True:
        print(' ')
        tool = questionary.select(
            '🪛 Do you use pip or poetry?',
            choices=[
                Choice('Poetry', 'poetry'),
                Choice('Pip', 'pip'),
                Choice('Other', 'other'),
            ],
        ).ask()

        if tool == 'poetry':
            try:
                subprocess.check_output(['poetry', 'add', *dependencies])
                print('😎 ALL DONE!')
                break
            except:
                print('Error using poetry\n')
                retry = questionary.confirm('🧐 Did you want use pip?').ask()
                if not retry:
                    print(
                        f'Install the packages using your tool: {" ".join(dependencies)}'
                    )
                    break
        elif tool == 'pip':
            subprocess.run(['pip', 'install', *dependencies], check=True)
            print('😎 ALL DONE!')
            break
        else:
            print(
                f'Install the packages using your tool: {" ".join(dependencies)}'
            )
